// Listing 3.7: Verwendung eigener TLS-Root-Zertifikate mit NSStream
// abgeleitet aus Listing 5 aus [Appl_13.16]


// Der folgende Code wird ausgefhrt, nachdem der Socket angelegt wurde:

[inStream setProperty:NSStreamSocketSecurityLevelNegotiatedSSL 
          forKey:NSStreamSocketSecurityLevelKey];

NSDictionary *sslSettings = [NSDictionary dictionaryWithObjectsAndKeys:
                             (id)kCFBooleanFalse, (id)kCFStreamSSLValidatesCertificateChain,
                             nil];
 
[inStream setProperty: sslSettings forKey: (__bridge NSString *)kCFStreamPropertySSLSettings];
 
 
...
 
// Methoden in der Delegations-Klasse des Stream:
  
NSString *kAnchorAlreadyAdded = @"AnchorAlreadyAdded";
 
- (void)stream:(NSStream *)theStream handleEvent:(NSStreamEvent)streamEvent
{
    if (streamEvent == NSStreamEventHasBytesAvailable || streamEvent == NSStreamEventHasSpaceAvailable) {
        // Platz auf dem Socket steht bereit, das Zertifikat kann geprft werden
 
        NSArray *certs = [theStream propertyForKey: (__bridge NSString *)kCFStreamPropertySSLPeerCertificates];
        SecTrustRef trust = (SecTrustRef)[theStream propertyForKey: (__bridge NSString *)kCFStreamPropertySSLPeerTrust];
        
        // Um das Array mit den Zertifikaten nich unntig aufzublhen wird 
        // das Zertifikat nur beim ersten Empfang von Daten zur Liste 
        // vertrauenswrdige Zertifikate hinzugefgt.
        NSNumber *alreadyAdded = [theStream propertyForKey: kAnchorAlreadyAdded];
        if (!alreadyAdded || ![alreadyAdded boolValue]) {
            trust = addAnchorToTrust(trust, self.trustedCert); // aus Listing 3.3.
            [theStream setProperty: [NSNumber numberWithBool: YES] forKey: kAnchorAlreadyAdded];
        }
        SecTrustResultType res = kSecTrustResultInvalid;
 
        if (SecTrustEvaluate(trust, &res)) {
            // Die Prfung der Vertrauenswrdigkeit ist fehl geschlagen
            // Entweder ist das Zertifikat nicht in Ordnung oder es gibt 
            // irgend einen anderen Fehler
 
            // Abbauen des Input-Streams
            [theStream removeFromRunLoop: ... forMode: ...];
            [theStream setDelegate: nil];
            [theStream close];
 
            // Abbauen des Output-Streams
            ...
 
            return;
 
        }
 
        if (res != kSecTrustResultProceed && res != kSecTrustResultUnspecified) {
            // Der Host ist nicht vertrauenswrdig
            
            // Abbauen des Input-Streams
            [theStream removeFromRunLoop: ... forMode: ...];
            [theStream setDelegate: nil];
            [theStream close];
 
            // Abbauen des Output-Streams
            ...
 
        } else {
            // Der Host is vertrauenswrdig, die Daten knnen normal 
            // verarbeitet werden
            
        }
    }
}
